@@ -1,4 +1,4 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 
 """
 Copyright (c) 2006-2019 sqlmap developers (http://sqlmap.org/)
@@ -10,16 +10,16 @@ import os
 import re
 import shutil
 import subprocess
-import sys
 import time
-import urllib
 import zipfile
 
 from lib.core.common import dataToStdout
 from lib.core.common import getSafeExString
 from lib.core.common import getLatestRevision
+from lib.core.common import openFile
 from lib.core.common import pollProcess
 from lib.core.common import readInput
+from lib.core.convert import getText
 from lib.core.data import conf
 from lib.core.data import logger
 from lib.core.data import paths
@@ -28,7 +28,7 @@ from lib.core.settings import GIT_REPOSITORY
 from lib.core.settings import IS_WIN
 from lib.core.settings import VERSION
 from lib.core.settings import ZIPBALL_PAGE
-from lib.core.settings import UNICODE_ENCODING
+from thirdparty.six.moves import urllib as _urllib
 
 def update():
     if not conf.updateAll:
@@ -71,7 +71,7 @@ def update():
                     logger.error(errMsg)
                 else:
                     try:
-                        archive = urllib.urlretrieve(ZIPBALL_PAGE)[0]
+                        archive = _urllib.request.urlretrieve(ZIPBALL_PAGE)[0]
 
                         with zipfile.ZipFile(archive) as f:
                             for info in f.infolist():
@@ -81,7 +81,7 @@ def update():
 
                         filepath = os.path.join(paths.SQLMAP_ROOT_PATH, "lib", "core", "settings.py")
                         if os.path.isfile(filepath):
-                            with open(filepath, "rb") as f:
+                            with openFile(filepath, "rb") as f:
                                 version = re.search(r"(?m)^VERSION\s*=\s*['\"]([^'\"]+)", f.read()).group(1)
                                 logger.info("updated to the latest version '%s#dev'" % version)
                                 success = True
@@ -106,23 +106,25 @@ def update():
         dataToStdout("\r[%s] [INFO] update in progress" % time.strftime("%X"))
 
         try:
-            process = subprocess.Popen("git checkout . && git pull %s HEAD" % GIT_REPOSITORY, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, cwd=paths.SQLMAP_ROOT_PATH.encode(sys.getfilesystemencoding() or UNICODE_ENCODING))
+            process = subprocess.Popen("git checkout . && git pull %s HEAD" % GIT_REPOSITORY, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, cwd=paths.SQLMAP_ROOT_PATH)
             pollProcess(process, True)
-            stdout, stderr = process.communicate()
+            output, _ = process.communicate()
             success = not process.returncode
-        except (IOError, OSError) as ex:
+        except Exception as ex:
             success = False
-            stderr = getSafeExString(ex)
+            output = getSafeExString(ex)
+        finally:
+            output = getText(output)
 
         if success:
-            logger.info("%s the latest revision '%s'" % ("already at" if "Already" in stdout else "updated to", getRevisionNumber()))
+            logger.info("%s the latest revision '%s'" % ("already at" if "Already" in output else "updated to", getRevisionNumber()))
         else:
-            if "Not a git repository" in stderr:
+            if "Not a git repository" in output:
                 errMsg = "not a valid git repository. Please checkout the 'sqlmapproject/sqlmap' repository "
                 errMsg += "from GitHub (e.g. 'git clone --depth 1 %s sqlmap')" % GIT_REPOSITORY
                 logger.error(errMsg)
             else:
-                logger.error("update could not be completed ('%s')" % re.sub(r"\W+", " ", stderr).strip())
+                logger.error("update could not be completed ('%s')" % re.sub(r"\W+", " ", output).strip())
 
     if not success:
         if IS_WIN:
