@@ -11,6 +11,7 @@ import re
 from xml.etree import ElementTree as et
 
 from lib.core.common import getSafeExString
+from lib.core.compat import xrange
 from lib.core.data import conf
 from lib.core.data import paths
 from lib.core.datatype import AttribDict
@@ -24,8 +25,8 @@ def cleanupVals(text, tag):
     if tag in ("clause", "where"):
         text = text.split(',')
 
-    if isinstance(text, basestring):
-        text = int(text) if text.isdigit() else text
+    if hasattr(text, "isdigit") and text.isdigit():
+        text = int(text)
 
     elif isinstance(text, list):
         count = 0
