@@ -1,4 +1,4 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 
 """
 Copyright (c) 2006-2019 sqlmap developers (http://sqlmap.org/)
@@ -34,13 +34,11 @@ from lib.core.common import evaluateCode
 from lib.core.common import extractRegexResult
 from lib.core.common import filterNone
 from lib.core.common import findMultipartPostBoundary
-from lib.core.common import getBytes
 from lib.core.common import getCurrentThreadData
 from lib.core.common import getHeader
 from lib.core.common import getHostHeader
 from lib.core.common import getRequestHeader
 from lib.core.common import getSafeExString
-from lib.core.common import getUnicode
 from lib.core.common import isMultiThreadMode
 from lib.core.common import logHTTPTraffic
 from lib.core.common import pushValue
@@ -58,7 +56,10 @@ from lib.core.common import wasLastResponseDelayed
 from lib.core.common import unsafeVariableNaming
 from lib.core.common import urldecode
 from lib.core.common import urlencode
+from lib.core.compat import patchHeaders
 from lib.core.compat import xrange
+from lib.core.convert import getBytes
+from lib.core.convert import getUnicode
 from lib.core.data import conf
 from lib.core.data import kb
 from lib.core.data import logger
@@ -123,6 +124,7 @@ from lib.request.comparison import comparison
 from lib.request.methodrequest import MethodRequest
 from thirdparty import six
 from thirdparty.odict import OrderedDict
+from thirdparty.six import unichr as _unichr
 from thirdparty.six.moves import http_client as _http_client
 from thirdparty.six.moves import urllib as _urllib
 from thirdparty.socks.socks import ProxyError
@@ -144,7 +146,7 @@ class Connect(object):
         threadData = getCurrentThreadData()
         threadData.retriesCount += 1
 
-        if conf.proxyList and threadData.retriesCount >= conf.retries:
+        if conf.proxyList and threadData.retriesCount >= conf.retries and not kb.locks.handlers.locked():
             warnMsg = "changing proxy"
             logger.warn(warnMsg)
 
@@ -234,22 +236,8 @@ class Connect(object):
         the target URL page content
         """
 
-        start = time.time()
-
-        if isinstance(conf.delay, (int, float)) and conf.delay > 0:
-            time.sleep(conf.delay)
-
         if conf.offline:
             return None, None, None
-        elif conf.dummy or conf.murphyRate and randomInt() % conf.murphyRate == 0:
-            if conf.murphyRate:
-                time.sleep(randomInt() % (MAX_MURPHY_SLEEP_TIME + 1))
-            return getUnicode(randomStr(int(randomInt()), alphabet=[chr(_) for _ in xrange(256)]), {}, int(randomInt())), None, None if not conf.murphyRate else randomInt(3)
-
-        threadData = getCurrentThreadData()
-        with kb.locks.request:
-            kb.requestCounter += 1
-            threadData.lastRequestUID = kb.requestCounter
 
         url = kwargs.get("url", None) or conf.url
         get = kwargs.get("get", None)
@@ -275,15 +263,36 @@ class Connect(object):
         finalCode = kwargs.get("finalCode", False)
         chunked = kwargs.get("chunked", False) or conf.chunked
 
+        start = time.time()
+
+        if isinstance(conf.delay, (int, float)) and conf.delay > 0:
+            time.sleep(conf.delay)
+
+        threadData = getCurrentThreadData()
+        with kb.locks.request:
+            kb.requestCounter += 1
+            threadData.lastRequestUID = kb.requestCounter
+
+        if conf.dummy or conf.murphyRate and randomInt() % conf.murphyRate == 0:
+            if conf.murphyRate:
+                time.sleep(randomInt() % (MAX_MURPHY_SLEEP_TIME + 1))
+
+            page, headers, code = randomStr(int(randomInt()), alphabet=[_unichr(_) for _ in xrange(256)]), None, None if not conf.murphyRate else randomInt(3)
+
+            threadData.lastPage = page
+            threadData.lastCode = code
+
+            return page, headers, code
+
         if multipart:
             post = multipart
+        else:
+            if not post:
+                chunked = False
 
-        if not post:
-            chunked = False
-
-        elif chunked:
-            post = _urllib.parse.unquote(post)
-            post = chunkSplitPostData(post)
+            elif chunked:
+                post = _urllib.parse.unquote(post)
+                post = chunkSplitPostData(post)
 
         websocket_ = url.lower().startswith("ws")
 
@@ -415,9 +424,13 @@ class Connect(object):
 
             for key, value in list(headers.items()):
                 del headers[key]
-                for char in (r"\r", r"\n"):
-                    value = re.sub(r"(%s)([^ \t])" % char, r"\g<1>\t\g<2>", value)
-                headers[getBytes(key)] = getBytes(value.strip("\r\n"))
+                if isinstance(value, six.string_types):
+                    for char in (r"\r", r"\n"):
+                        value = re.sub(r"(%s)([^ \t])" % char, r"\g<1>\t\g<2>", value)
+                    headers[getBytes(key) if six.PY2 else key] = getBytes(value.strip("\r\n"))  # Note: Python3 has_header() expects non-bytes value
+
+            if six.PY2:
+                url = getBytes(url)  # Note: Python3 requires text while Python2 has problems when mixing text with binary POST
 
             post = getBytes(post)
 
@@ -450,7 +463,6 @@ class Connect(object):
                 logger.log(CUSTOM_LOGGING.TRAFFIC_OUT, requestMsg)
             else:
                 if method and method not in (HTTPMETHOD.GET, HTTPMETHOD.POST):
-                    method = getBytes(method)
                     req = MethodRequest(url, post, headers)
                     req.set_method(method)
                 elif url is not None:
@@ -517,6 +529,7 @@ class Connect(object):
                     code = (code or conn.code) if conn.code == kb.originalCode else conn.code  # do not override redirection code (for comparison purposes)
                     responseHeaders = conn.info()
                     responseHeaders[URI_HTTP_HEADER] = conn.geturl()
+                    patchHeaders(responseHeaders)
                     kb.serverHeader = responseHeaders.get(HTTP_HEADER.SERVER, kb.serverHeader)
                 else:
                     code = None
@@ -592,6 +605,7 @@ class Connect(object):
                 page = ex.read() if not skipRead else None
                 responseHeaders = ex.info()
                 responseHeaders[URI_HTTP_HEADER] = ex.geturl()
+                patchHeaders(responseHeaders)
                 page = decodePage(page, responseHeaders.get(HTTP_HEADER.CONTENT_ENCODING), responseHeaders.get(HTTP_HEADER.CONTENT_TYPE))
             except socket.timeout:
                 warnMsg = "connection timed out while trying "
@@ -606,7 +620,7 @@ class Connect(object):
                 page = getUnicode(page)
 
             code = ex.code
-            status = getattr(ex, "reason", None) or getSafeExString(ex).split(": ", 1)[-1]
+            status = getUnicode(getattr(ex, "reason", None) or getSafeExString(ex).split(": ", 1)[-1])
 
             kb.originalCode = kb.originalCode or code
             threadData.lastHTTPError = (threadData.lastRequestUID, code, status)
@@ -668,6 +682,9 @@ class Connect(object):
         except (_urllib.error.URLError, socket.error, socket.timeout, _http_client.HTTPException, struct.error, binascii.Error, ProxyError, SqlmapCompressionException, WebSocketException, TypeError, ValueError, OverflowError):
             tbMsg = traceback.format_exc()
 
+            if conf.debug:
+                dataToStdout(tbMsg)
+
             if checking:
                 return None, None, None
             elif "no host given" in tbMsg:
@@ -691,7 +708,7 @@ class Connect(object):
                 warnMsg = "connection reset to the target URL"
             elif "URLError" in tbMsg or "error" in tbMsg:
                 warnMsg = "unable to connect to the target URL"
-                match = re.search(r"Errno \d+\] ([^>]+)", tbMsg)
+                match = re.search(r"Errno \d+\] ([^>\n]+)", tbMsg)
                 if match:
                     warnMsg += " ('%s')" % match.group(1).strip()
             elif "NTLM" in tbMsg:
@@ -778,7 +795,7 @@ class Connect(object):
 
             socket.setdefaulttimeout(conf.timeout)
 
-        processResponse(page, responseHeaders, status)
+        processResponse(page, responseHeaders, code, status)
 
         if not skipLogTraffic:
             if conn and getattr(conn, "redurl", None):
@@ -796,7 +813,7 @@ class Connect(object):
                 responseMsg += "[#%d] (%s %s):\r\n" % (threadData.lastRequestUID, code, status)
 
             if responseHeaders:
-                logHeaders = getUnicode("".join(responseHeaders.headers).strip() if six.PY2 else responseHeaders.__bytes__())
+                logHeaders = getUnicode("".join(responseHeaders.headers).strip())
 
             logHTTPTraffic(requestMsg, "%s%s\r\n\r\n%s" % (responseMsg, logHeaders, (page or "")[:MAX_CONNECTION_CHUNK_SIZE]), start, time.time())
 
@@ -1034,7 +1051,7 @@ class Connect(object):
 
                 match = re.search(r"String\.fromCharCode\(([\d+, ]+)\)", token.value)
                 if match:
-                    token.value = "".join(chr(int(_)) for _ in match.group(1).replace(' ', "").split(','))
+                    token.value = "".join(_unichr(int(_)) for _ in match.group(1).replace(' ', "").split(','))
 
             if not token:
                 if conf.csrfUrl and conf.csrfToken and conf.csrfUrl != conf.url and code == _http_client.OK:
@@ -1065,11 +1082,11 @@ class Connect(object):
             if token:
                 token.value = token.value.strip("'\"")
 
-                for place in (PLACE.GET, PLACE.POST):
-                    if place in conf.parameters:
-                        if place == PLACE.GET and get:
+                for candidate in (PLACE.GET, PLACE.POST):
+                    if candidate in conf.parameters:
+                        if candidate == PLACE.GET and get:
                             get = _adjustParameter(get, token.name, token.value)
-                        elif place == PLACE.POST and post:
+                        elif candidate == PLACE.POST and post:
                             post = _adjustParameter(post, token.name, token.value)
 
                 for i in xrange(len(conf.httpHeaders)):
@@ -1349,6 +1366,8 @@ class Connect(object):
             kb.permissionFlag = True
             singleTimeWarnMessage("potential permission problems detected ('%s')" % message)
 
+        patchHeaders(headers)
+
         if content or response:
             return page, headers, code
 
