@@ -1,4 +1,4 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 
 """
 Copyright (c) 2006-2019 sqlmap developers (http://sqlmap.org/)
@@ -29,7 +29,7 @@ class Enumeration(GenericEnumeration):
 
         return kb.data.banner
 
-    def getPrivileges(self, *args):
+    def getPrivileges(self, *args, **kwargs):
         warnMsg = "on H2 it is not possible to enumerate the user privileges"
         logger.warn(warnMsg)
 
@@ -47,3 +47,9 @@ class Enumeration(GenericEnumeration):
         logger.warn(warnMsg)
 
         return {}
+
+    def getStatements(self):
+        warnMsg = "on H2 it is not possible to enumerate the SQL statements"
+        logger.warn(warnMsg)
+
+        return []
