@@ -1,4 +1,4 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 
 """
 Copyright (c) 2006-2019 sqlmap developers (http://sqlmap.org/)
@@ -21,6 +21,8 @@ warnings.filterwarnings(action="ignore", category=DeprecationWarning)
 from sqlmap import modulePath
 from lib.core.common import setPaths
 from lib.core.data import logger
+from lib.core.patch import dirtyPatches
+from lib.core.patch import resolveCrossReferences
 from lib.core.settings import RESTAPI_DEFAULT_ADAPTER
 from lib.core.settings import RESTAPI_DEFAULT_ADDRESS
 from lib.core.settings import RESTAPI_DEFAULT_PORT
@@ -32,6 +34,9 @@ def main():
     REST-JSON API main function
     """
 
+    dirtyPatches()
+    resolveCrossReferences()
+
     # Set default logging level to debug
     logger.setLevel(logging.DEBUG)
 
